/*-------------------------------------------------------------------------*/
/* PFF - Low level disk control module for PIC            (C)ChaN, 2010    */
/* This is just the low-level spi stuff                                     */
/*-------------------------------------------------------------------------*/
// See: http://elm-chan.org/docs/mmc/mmc_e.html
//
#include <xc.h>			/* Device include file */
#include <stdint.h>
#include "ff.h"
#include "diskio.h"
#include "defines.h"
//
// https://github.com/mondul/fatfs_pic18
//
/*-------------------------------------------------------------------------*/
/* Platform dependent macros and functions needed to be modified           */
/*-------------------------------------------------------------------------*/

#define SELECT  	(PORTCbits.RC0 = 0)		/* CS = L */
#define	DESELECT	(PORTCbits.RC0 = 1)		/* CS = H */
#define	MMC_SEL		!PORTCbits.RC0		/* CS status (true:CS == L) */
#define FCY 16000000

void SD_CS_SetHigh(void)        /* Set CS# high */
{
    PORTCbits.RC0 = 1;
}

void SD_CS_SetLow(void)
{
    PORTCbits.RC0 = 0;
}

void init_spi (char speed)		/* Initialize SPI port */
{
    if (speed)
    {
        /* Set-up SPI bus, fast*/
        SSP1CON1 = 0x00; //Disable SPI
        //SSPCON1 = 0x00; //CKP = 0, SSPM = 0000, SPI Master, clock = Fosc/4 (= 8 MHz)
		//SSPCON1 = 0x01; //CKP = 0, SSPM = 0001, SPI Master, clock = Fosc/16 (= 2 MHz)
        SSP1ADD = 0x4F;
		//SSP1CON1 = 0x10; // SSPDIV; //CKP = 0, SSPM = 0001, SPI Master, clock = FOSC/(4*(SSPxADD+1)) (= 2 MHz)
        SSP1CON1 = 0x2A;
        SSP1STAT = 0x40;  //CKE = 1, Data transmitted on falling edge
        SSP1CON1 |= (1<<5); //SSPEN = 1, Enable SPI bus

    }
    else
    {
        /* Set-up SPI bus, slow*/
        SSP1CON1 = 0x00; //Disable SPI
        SSP1CON1 = 0x10; //CKP = 0, SSPM = 0010, SPI Master, clock = FOSC/(4*(SSPxADD+1)) (= 500 kHz)
        SSP1CON1 = 0x2A;
        SSP1ADD = 0xF0; // was 0x90
        SSP1STAT = 0x40;  //CKE = 1, Data transmitted on falling edge
        //SSP1STAT = 0x00;  //CKE = 0, Data transmitted on falling edge
        SSP1CON1 |= (1<<5); //SSPEN = 1, Enable SPI bus
    }

}

//void dly_100us (void)		/* Delay 100 microseconds */
//{
//	UINT n = FCY / 100000;
//	do {
//		LATA; LATA; LATA; LATA; LATA; LATA;
//	} while (--n);
//}

void xmit_spi (BYTE d)		/* Send a byte to the MMC */
{
    SSP1BUF = (WORD)d;  //Send data
    while(SSP1STATbits.BF);  //While full wait
    while(!SSP1STATbits.BF);  //While full wait
    d = SSP1BUF;
}

BYTE rcv_spi (void)			/* Send a 0xFF to the MMC and get the received byte */
{

    SSP1BUF = 0x00FF;
    while(SSP1STATbits.BF);  //While full wait
    while(!SSP1STATbits.BF);  //While full wait
    return ((BYTE)SSP1BUF);
    /*
    unsigned char data;
    PIR1bits.SSPIF = 0; //Make certain interrupt flag is clear
    data = SSPBUF;         // Make certain BF is clear.
    SSPBUF = 0xFF;            // initiate bus cycle
    while(!PIR1bits.SSPIF);   // wait until bus cycle complete
    return(SSPBUF);             // return with byte read
    */
}